@@ -25,7 +25,9 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
+#ifdef ENABLE_GSPELL
 #include <gspell/gspell.h>
+#endif
 
 #include "gr-details-page.h"
 #include "gr-recipe-store.h"
@@ -66,7 +68,7 @@ timer_complete (GrTimer *timer)
 
         store = gr_app_get_recipe_store (GR_APP (app));
         recipe = gr_recipe_store_get_recipe (store, id);
-        name = gr_recipe_get_name (recipe);
+        name = gr_recipe_get_translated_name (recipe);
 
         body = g_strdup_printf (_("Your cooking timer for “%s” has expired."), name);
 
@@ -124,6 +126,8 @@ struct _GrDetailsPage
         GtkWidget *warning_box;
         GtkWidget *spicy_warning;
         GtkWidget *garlic_warning;
+        GtkWidget *gluten_warning;
+        GtkWidget *dairy_warning;
         GtkWidget *ingredients_box;
         GtkWidget *instructions_label;
         GtkWidget *cooking_revealer;
@@ -143,8 +147,13 @@ struct _GrDetailsPage
         GtkWidget *delete_button;
         GtkWidget *notes_field;
         GtkWidget *description_label;
+        GtkWidget *export_button;
+        GtkWidget *error_label;
+        GtkWidget *error_revealer;
 
         guint save_timeout;
+
+        char *uri;
 };
 
 G_DEFINE_TYPE (GrDetailsPage, gr_details_page, GTK_TYPE_BOX)
@@ -178,7 +187,7 @@ timer_remaining_changed (GrTimer       *timer,
         guint hours, minutes, seconds;
         g_autofree char *buf = NULL;
 
-        if (strcmp (gr_timer_get_name (timer), gr_recipe_get_name (page->recipe)) != 0)
+        if (strcmp (gr_timer_get_name (timer), gr_recipe_get_id (page->recipe)) != 0)
                 return;
 
         remaining = gr_timer_get_remaining (timer);
@@ -284,6 +293,9 @@ print_recipe (GrDetailsPage *page)
 static void
 export_recipe (GrDetailsPage *page)
 {
+        g_autofree char *dir = NULL;
+        g_autofree char *basename = NULL;
+        g_autofree char *path = NULL;
         g_autoptr(GFile) file = NULL;
 
         if (!page->exporter) {
@@ -293,7 +305,13 @@ export_recipe (GrDetailsPage *page)
                 page->exporter = gr_recipe_exporter_new (GTK_WINDOW (window));
         }
 
-        file = g_file_new_for_path (g_get_tmp_dir ());
+        dir = g_dir_make_tmp ("recipesXXXXXX", NULL);
+        basename = g_strconcat (gr_recipe_get_name (page->recipe), ".tar.gz", NULL);
+        if (dir)
+                path = g_build_filename (dir, basename, NULL);
+        else
+                path = g_strdup (basename);
+        file = g_file_new_for_path (path);
         gr_recipe_exporter_export_to (page->exporter, page->recipe, file);
 }
 
@@ -492,6 +510,7 @@ details_page_finalize (GObject *object)
         g_clear_object (&self->printer);
         g_clear_object (&self->exporter);
         g_clear_pointer (&self->cooking, g_hash_table_unref);
+        g_clear_pointer (&self->uri, g_free);
 
         if (self->save_timeout) {
                 g_source_remove (self->save_timeout);
@@ -577,11 +596,69 @@ schedule_save (GtkTextBuffer *buffer, GrDetailsPage *page)
         page->save_timeout = g_timeout_add (250, save_notes, page);
 }
 
+static gboolean
+activate_uri_at_idle (gpointer data)
+{
+        GrDetailsPage *page = data;
+        g_autofree char *uri = NULL;
+
+        uri = page->uri;
+        page->uri = NULL;
+
+        if (g_str_has_prefix (uri, "image:")) {
+                int idx;
+
+                idx = (int)g_ascii_strtoll (uri + strlen ("image:"), NULL, 10);
+                gr_image_viewer_show_image (GR_IMAGE_VIEWER (page->recipe_image), idx);
+        }
+        else if (g_str_has_prefix (uri, "recipe:")) {
+                GrRecipeStore *store;
+                const char *id;
+                g_autoptr(GrRecipe) recipe = NULL;
+
+                store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+
+                id = uri + strlen ("recipe:");
+                recipe = gr_recipe_store_get_recipe (store, id);
+                if (recipe) {
+                        GtkWidget *window;
+
+                        window = gtk_widget_get_ancestor (GTK_WIDGET (page), GTK_TYPE_APPLICATION_WINDOW);
+                        gr_window_show_recipe (GR_WINDOW (window), recipe);
+                }
+                else {
+                        gtk_label_set_label (GTK_LABEL (page->error_label),
+                                             _("Could not find this recipe."));
+                        gtk_revealer_set_reveal_child (GTK_REVEALER (page->error_revealer), TRUE);
+                }
+        }
+
+        return G_SOURCE_REMOVE;
+}
+
+static gboolean
+activate_link (GtkLabel      *label,
+               const char    *uri,
+               GrDetailsPage *page)
+{
+        g_free (page->uri);
+        page->uri = g_strdup (uri);
+
+        // FIXME: We can avoid the idle with GTK+ 3.22.6 or newer
+        g_idle_add (activate_uri_at_idle, page);
+
+        return TRUE;
+}
+
+static void
+dismiss_error (GrDetailsPage *page)
+{
+        gtk_revealer_set_reveal_child (GTK_REVEALER (page->error_revealer), FALSE);
+}
+
 static void
 gr_details_page_init (GrDetailsPage *page)
 {
-        GspellTextView *gspell_view;
-
         gtk_widget_set_has_window (GTK_WIDGET (page), FALSE);
         gtk_widget_init_template (GTK_WIDGET (page));
         connect_store_signals (page);
@@ -597,8 +674,17 @@ gr_details_page_init (GrDetailsPage *page)
 
         g_signal_connect (gtk_text_view_get_buffer (GTK_TEXT_VIEW (page->notes_field)), "changed", G_CALLBACK (schedule_save), page);
 
-        gspell_view = gspell_text_view_get_from_gtk_text_view (GTK_TEXT_VIEW (page->notes_field));
-        gspell_text_view_basic_setup (gspell_view);
+#ifdef ENABLE_GSPELL
+        {
+                GspellTextView *gspell_view;
+                gspell_view = gspell_text_view_get_from_gtk_text_view (GTK_TEXT_VIEW (page->notes_field));
+                gspell_text_view_basic_setup (gspell_view);
+        }
+#endif
+
+#ifdef ENABLE_AUTOAR
+        gtk_widget_show (page->export_button);
+#endif
 }
 
 static void
@@ -618,6 +704,8 @@ gr_details_page_class_init (GrDetailsPageClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, warning_box);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, spicy_warning);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, garlic_warning);
+        gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, dairy_warning);
+        gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, gluten_warning);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, ingredients_box);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, instructions_label);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, cooking_revealer);
@@ -637,6 +725,9 @@ gr_details_page_class_init (GrDetailsPageClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, delete_button);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, notes_field);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, description_label);
+        gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, export_button);
+        gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, error_label);
+        gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, error_revealer);
 
         gtk_widget_class_bind_template_callback (widget_class, edit_recipe);
         gtk_widget_class_bind_template_callback (widget_class, delete_recipe);
@@ -649,6 +740,8 @@ gr_details_page_class_init (GrDetailsPageClass *klass)
         gtk_widget_class_bind_template_callback (widget_class, time_spin_output);
         gtk_widget_class_bind_template_callback (widget_class, check_clicked);
         gtk_widget_class_bind_template_callback (widget_class, cook_it_later);
+        gtk_widget_class_bind_template_callback (widget_class, activate_link);
+        gtk_widget_class_bind_template_callback (widget_class, dismiss_error);
 }
 
 GtkWidget *
@@ -668,7 +761,6 @@ populate_ingredients (GrDetailsPage *page,
 {
         g_autoptr(GtkSizeGroup) group = NULL;
         g_autofree char **segments = NULL;
-        g_auto(GStrv) ings = NULL;
         int i, j;
         GtkWidget *list;
         GtkWidget *label;
@@ -678,6 +770,8 @@ populate_ingredients (GrDetailsPage *page,
         group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
         segments = gr_ingredients_list_get_segments (page->ingredients);
         for (j = 0; segments[j]; j++) {
+                g_auto(GStrv) ings = NULL;
+
                 if (segments[j] && segments[j][0]) {
                         label = gtk_label_new (segments[j]);
                         gtk_widget_show (label);
@@ -717,8 +811,8 @@ populate_ingredients (GrDetailsPage *page,
                         g_object_set (label,
                                       "visible", TRUE,
                                       "xalign", 0.0,
-                              "margin", 10,
-                              NULL);
+                                      "margin", 10,
+                                      NULL);
                         gtk_container_add (GTK_CONTAINER (box), label);
 
                         gtk_container_add (GTK_CONTAINER (list), box);
@@ -729,12 +823,22 @@ populate_ingredients (GrDetailsPage *page,
 
         gtk_widget_hide (page->warning_box);
         gtk_widget_hide (page->garlic_warning);
+        gtk_widget_hide (page->dairy_warning);
+        gtk_widget_hide (page->gluten_warning);
         gtk_widget_hide (page->spicy_warning);
 
         if (gr_recipe_contains_garlic (page->recipe)) {
                 gtk_widget_show (page->warning_box);
                 gtk_widget_show (page->garlic_warning);
         }
+        if (gr_recipe_contains_dairy (page->recipe)) {
+                gtk_widget_show (page->warning_box);
+                gtk_widget_show (page->dairy_warning);
+        }
+        if (gr_recipe_contains_gluten (page->recipe)) {
+                gtk_widget_show (page->warning_box);
+                gtk_widget_show (page->gluten_warning);
+        }
 
         if (gr_recipe_get_spiciness (page->recipe) > 50) {
                 gtk_widget_show (page->warning_box);
@@ -765,7 +869,6 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         const char *instructions;
         const char *notes;
         const char *description;
-        g_autoptr(GdkPixbuf) pixbuf = NULL;
         GrRecipeStore *store;
         g_autoptr(GrChef) chef = NULL;
         g_autoptr(GrIngredientsList) ing = NULL;
@@ -781,9 +884,9 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         prep_time = gr_recipe_get_prep_time (recipe);
         cook_time = gr_recipe_get_cook_time (recipe);
         ingredients = gr_recipe_get_ingredients (recipe);
-        instructions = gr_recipe_get_instructions (recipe);
         notes = gr_recipe_get_notes (recipe);
-        description = gr_recipe_get_description (recipe);
+        instructions = gr_recipe_get_translated_instructions (recipe);
+        description = gr_recipe_get_translated_description (recipe);
 
         g_object_get (recipe, "images", &images, NULL);
         gr_image_viewer_set_images (GR_IMAGE_VIEWER (page->recipe_image), images);
@@ -793,9 +896,17 @@ gr_details_page_set_recipe (GrDetailsPage *page,
 
         populate_ingredients (page, serves, serves);
 
-        gtk_label_set_label (GTK_LABEL (page->prep_time_label), prep_time);
-        gtk_label_set_label (GTK_LABEL (page->cook_time_label), cook_time);
+        if (prep_time[0] == '\0')
+                gtk_label_set_label (GTK_LABEL (page->prep_time_label), "");
+        else
+                gtk_label_set_label (GTK_LABEL (page->prep_time_label), _(prep_time));
+
+        if (cook_time[0] == '\0')
+                gtk_label_set_label (GTK_LABEL (page->cook_time_label), "");
+        else
+                gtk_label_set_label (GTK_LABEL (page->cook_time_label), _(cook_time));
         gtk_label_set_label (GTK_LABEL (page->instructions_label), instructions);
+        gtk_label_set_track_visited_links (GTK_LABEL (page->instructions_label), FALSE);
 
         gtk_spin_button_set_value (GTK_SPIN_BUTTON (page->serves_spin), serves);
         gtk_widget_set_sensitive (page->serves_spin, ing != NULL);
@@ -845,6 +956,12 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         }
 }
 
+GrRecipe *
+gr_details_page_get_recipe (GrDetailsPage *page)
+{
+        return page->recipe;
+}
+
 static void
 details_page_reload (GrDetailsPage *page,
                      GrRecipe      *recipe)
@@ -852,8 +969,8 @@ details_page_reload (GrDetailsPage *page,
         const char *name;
         const char *new_name;
 
-        name = gr_recipe_get_name (page->recipe);
-        new_name = gr_recipe_get_name (recipe);
+        name = gr_recipe_get_id (page->recipe);
+        new_name = gr_recipe_get_id (recipe);
         if (strcmp (name, new_name) == 0)
                 gr_details_page_set_recipe (page, recipe);
 }
