@@ -23,7 +23,9 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
+#ifdef ENABLE_GSPELL
 #include <gspell/gspell.h>
+#endif
 
 #include "gr-edit-page.h"
 #include "gr-recipe-store.h"
@@ -96,6 +98,15 @@ struct _GrEditPage
         GtkWidget *amount_search_button;
         GtkWidget *amount_search_button_label;
         GtkWidget *amount_search_revealer;
+        GtkWidget *recipe_popover;
+        GtkWidget *recipe_list;
+        GtkWidget *recipe_filter_entry;
+        GtkWidget *add_recipe_button;
+        GtkWidget *link_image_button;
+        GtkWidget *image_popover;
+        GtkWidget *image_flowbox;
+
+        GrRecipeSearch *search;
 
         GtkSizeGroup *group;
 
@@ -106,6 +117,7 @@ struct _GrEditPage
 
         char *ing_term;
         char *unit_term;
+        char *recipe_term;
 };
 
 G_DEFINE_TYPE (GrEditPage, gr_edit_page, GTK_TYPE_BOX)
@@ -117,7 +129,33 @@ dismiss_error (GrEditPage *page)
 }
 
 static void
-images_changed (GrEditPage *page)
+populate_image_flowbox (GrEditPage *page)
+{
+        int i;
+        g_autoptr(GArray) images = NULL;
+
+        g_object_get (page->images, "images", &images, NULL);
+
+        container_remove_all (GTK_CONTAINER (page->image_flowbox));
+
+        for (i = 0; i < images->len; i++) {
+                GrRotatedImage *ri = &g_array_index (images, GrRotatedImage, i);
+                g_autoptr(GdkPixbuf) pb = load_pixbuf_fill_size (ri->path, ri->angle, 60, 40);
+                GtkWidget *image;
+                GtkWidget *child;
+
+                image = gtk_image_new_from_pixbuf (pb);
+                gtk_widget_show (image);
+                gtk_container_add (GTK_CONTAINER (page->image_flowbox), image);
+                child = gtk_widget_get_parent (image);
+                g_object_set_data (G_OBJECT (child), "image-idx", GINT_TO_POINTER (i));
+        }
+}
+
+static void update_link_button_sensitivity (GrEditPage *page);
+
+static void
+update_image_button_sensitivity (GrEditPage *page)
 {
         g_autoptr(GArray) images = NULL;
         int length;
@@ -130,6 +168,14 @@ images_changed (GrEditPage *page)
         gtk_widget_set_sensitive (page->rotate_image_right_button, length > 0);
 }
 
+static void
+images_changed (GrEditPage *page)
+{
+        update_image_button_sensitivity (page);
+        update_link_button_sensitivity (page);
+        populate_image_flowbox (page);
+}
+
 static void
 add_image (GrEditPage *page)
 {
@@ -165,6 +211,9 @@ edit_page_finalize (GObject *object)
 
         g_free (self->ing_term);
         g_free (self->unit_term);
+        g_free (self->recipe_term);
+
+        g_clear_object (&self->search);
 
         G_OBJECT_CLASS (gr_edit_page_parent_class)->finalize (object);
 }
@@ -459,11 +508,6 @@ format_unit_for_display (const char *amount,
                          char **amount_out,
                          char **unit_out)
 {
-        g_autoptr(GrIngredientsList) ingredients = NULL;
-        g_autofree char *line = NULL;
-        g_autofree char *parsed = NULL;
-        g_auto(GStrv) strv = NULL;
-
         if (amount[0] != '\0') {
                 GrNumber number;
                 char *a = (char *)amount;
@@ -494,7 +538,6 @@ add_ingredient2 (GtkButton *button, GrEditPage *page)
         const char *ingredient;
         g_autofree char *amount = NULL;
         g_autofree  char *unit = NULL;
-        g_autofree char *s = NULL;
         GtkWidget *list;
         GtkWidget *b;
         GtkWidget *row;
@@ -534,13 +577,22 @@ collect_ingredients (GrEditPage *page)
                 children = gtk_container_get_children (GTK_CONTAINER (list));
                 for (l = children; l; l = l->next) {
                         GtkWidget *row = l->data;
+                        const char *amount;
+                        const char *unit;
+                        const char *ingredient;
+                        const char *id;
+
+                        amount = (const char *)g_object_get_data (G_OBJECT (row), "amount");
+                        unit = (const char *)g_object_get_data (G_OBJECT (row), "unit");
+                        ingredient = (const char *)g_object_get_data (G_OBJECT (row), "ingredient");
+                        id = gr_ingredient_get_id (ingredient);
                         if (s->len > 0)
                                 g_string_append (s, "\n");
-                        g_string_append (s, (const char *)g_object_get_data (G_OBJECT (row), "amount"));
+                        g_string_append (s, amount);
                         g_string_append (s, "\t");
-                        g_string_append (s, (const char *)g_object_get_data (G_OBJECT (row), "unit"));
+                        g_string_append (s, unit);
                         g_string_append (s, "\t");
-                        g_string_append (s, (const char *)g_object_get_data (G_OBJECT (row), "ingredient"));
+                        g_string_append (s, id ? id : ingredient);
                         g_string_append (s, "\t");
                         g_string_append (s, gtk_entry_get_text (GTK_ENTRY (entry)));
                 }
@@ -734,19 +786,18 @@ unit_filter_func (GtkListBoxRow *row,
 {
         GrEditPage *self = data;
         const char *unit;
-        const char *cf1, *cf2, *cf3;
+        g_autofree char *cf1 = NULL;
+        g_autofree char *cf2 = NULL;
 
         if (!self->unit_term)
                 return TRUE;
 
         unit = (const char *)g_object_get_data (G_OBJECT (row), "unit");
-        cf1 = gr_unit_get_abbreviation (unit);
-        cf2 = gr_unit_get_plural (unit);
-        cf3 = gr_unit_get_display_name (unit);
+        cf1 = g_utf8_casefold (gr_unit_get_abbreviation (unit), -1);
+        cf2 = g_utf8_casefold (gr_unit_get_display_name (unit), -1);
 
         return g_str_has_prefix (cf1, self->unit_term) ||
-               g_str_has_prefix (cf2, self->unit_term) ||
-               g_str_has_prefix (cf3, self->unit_term);
+               g_str_has_prefix (cf2, self->unit_term);
 }
 
 static void
@@ -826,11 +877,362 @@ populate_units_list (GrEditPage *self)
                           G_CALLBACK (unit_row_activated), self);
 }
 
+static gboolean
+recipe_filter_func (GtkListBoxRow *row,
+                    gpointer       data)
+{
+        GrEditPage *self = data;
+        const char *cf;
+        GrRecipe *r;
+
+        if (!self->recipe_term)
+                return TRUE;
+
+        r = GR_RECIPE (g_object_get_data (G_OBJECT (row), "recipe"));
+
+        if (self->recipe && strcmp (gr_recipe_get_id (r), gr_recipe_get_id (self->recipe)) == 0)
+                return FALSE;
+
+        cf = (const char *)g_object_get_data (G_OBJECT (row), "term");
+
+        return g_str_has_prefix (cf, self->recipe_term);
+}
+
+static void
+recipe_filter_changed (GrEditPage *self)
+{
+        const char *term;
+
+        term = gtk_entry_get_text (GTK_ENTRY (self->recipe_filter_entry));
+        g_free (self->recipe_term);
+        self->recipe_term = g_utf8_casefold (term, -1);
+        gtk_list_box_invalidate_filter (GTK_LIST_BOX (self->recipe_list));
+}
+
+static void
+recipe_filter_stop (GrEditPage *self)
+{
+}
+
+static void
+recipe_row_activated (GtkListBox    *list,
+                      GtkListBoxRow *row,
+                      GrEditPage    *self)
+{
+        GrRecipe *recipe;
+        const char *id;
+        const char *name;
+        g_autofree char *url = NULL;
+        GtkTextBuffer *buffer;
+        GtkTextIter start, end;
+        GtkTextTag *tag;
+        GdkRGBA color;
+
+        gdk_rgba_parse (&color, "blue");
+
+        recipe = GR_RECIPE (g_object_get_data (G_OBJECT (row), "recipe"));
+        id = gr_recipe_get_id (recipe);
+        name = gr_recipe_get_name (recipe);
+
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (self->instructions_field));
+        tag = gtk_text_buffer_create_tag (buffer, NULL,
+                                          "foreground-rgba", &color,
+                                          "underline", PANGO_UNDERLINE_SINGLE,
+                                          NULL);
+        url = g_strconcat ("recipe:", id, NULL);
+        g_object_set_data_full (G_OBJECT (tag), "href", g_strdup (url), g_free);
+
+        gtk_text_buffer_get_iter_at_mark (buffer, &start,
+                                          gtk_text_buffer_get_insert (buffer));
+        gtk_text_buffer_get_iter_at_mark (buffer, &end,
+                                          gtk_text_buffer_get_mark (buffer, "saved_selection_bound"));
+        gtk_text_buffer_delete (buffer, &start, &end);
+        gtk_text_buffer_insert (buffer, &start, "​", -1);
+        gtk_text_buffer_get_iter_at_mark (buffer, &start,
+                                          gtk_text_buffer_get_insert (buffer));
+        gtk_text_buffer_insert_with_tags (buffer, &start, name, -1, tag, NULL);
+
+        gtk_popover_popdown (GTK_POPOVER (self->recipe_popover));
+        gtk_entry_set_text (GTK_ENTRY (self->recipe_filter_entry), "");
+
+        gtk_widget_grab_focus (self->instructions_field);
+}
+
+static void
+recipe_filter_activated (GrEditPage *self)
+{
+        GtkListBoxRow *row;
+
+        row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (self->recipe_list), 0);
+        if (row) {
+                recipe_row_activated (GTK_LIST_BOX (self->recipe_list), row, self);
+        }
+}
+
+static void
+search_started (GrRecipeSearch *search,
+                GrEditPage     *page)
+{
+}
+
+static void
+search_hits_added (GrRecipeSearch *search,
+                   GList          *hits,
+                   GrEditPage     *page)
+{
+        GList *l;
+        GrRecipeStore *store;
+
+        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+
+        for (l = hits; l; l = l->next) {
+                GrRecipe *recipe = l->data;
+                GtkWidget *box;
+                GtkWidget *label;
+                GtkWidget *row;
+                const char *author;
+                char *tmp;
+                g_autoptr(GrChef) chef = NULL;
+
+                box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 20);
+                gtk_widget_show (box);
+                g_object_set (box,
+                              "margin-start", 20,
+                              "margin-end", 20,
+                              "margin-top", 10,
+                              "margin-bottom", 10,
+                              NULL);
+
+                label = gtk_label_new (gr_recipe_get_name (recipe));
+                gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+                gtk_widget_show (label);
+                gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
+
+                author = gr_recipe_get_author (recipe);
+                chef = gr_recipe_store_get_chef (store, author ? author : "");
+                if (chef) {
+                        tmp = g_strdup_printf (_("by %s"), gr_chef_get_name (chef));
+                        label = gtk_label_new (tmp);
+                        g_free (tmp);
+                        gtk_label_set_xalign (GTK_LABEL (label), 1.0);
+                        gtk_widget_show (label);
+                        gtk_style_context_add_class (gtk_widget_get_style_context (label), "dim-label");
+                        gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
+                }
+
+                gtk_container_add (GTK_CONTAINER (page->recipe_list), box);
+                row = gtk_widget_get_parent (box);
+                g_object_set_data_full (G_OBJECT (row), "recipe", g_object_ref (recipe), g_object_unref);
+                g_object_set_data_full (G_OBJECT (row), "term", g_utf8_casefold (gr_recipe_get_name (recipe), -1), g_free);
+        }
+}
+
+static void
+search_hits_removed (GrRecipeSearch *search,
+                     GList          *hits,
+                     GrEditPage     *page)
+{
+        GList *children, *l;
+
+        children = gtk_container_get_children (GTK_CONTAINER (page->recipe_list));
+        for (l = children; l; l = l->next) {
+                GtkWidget *row = l->data;
+                GrRecipe *recipe;
+
+                recipe = GR_RECIPE (g_object_get_data (G_OBJECT (row), "recipe"));
+                if (g_list_find (hits, recipe)) {
+                        gtk_container_remove (GTK_CONTAINER (page->recipe_list), row);
+                }
+        }
+}
+
+static void
+search_finished (GrRecipeSearch *search,
+                 GrEditPage     *page)
+{
+}
+
+static void
+update_link_button_sensitivity (GrEditPage *page)
+{
+        GtkTextBuffer *buffer;
+        GtkTextIter iter;
+        GSList *tags, *s;
+        gboolean in_link = FALSE;
+        g_autoptr(GArray) images = NULL;
+        int length;
+
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (page->instructions_field));
+
+        gtk_text_buffer_get_iter_at_mark (buffer, &iter,
+                                          gtk_text_buffer_get_insert (buffer));
+
+        tags = gtk_text_iter_get_tags (&iter);
+        for (s = tags; s; s = s->next) {
+                GtkTextTag *tag = s->data;
+
+                if (g_object_get_data (G_OBJECT (tag), "href") != NULL) {
+                        in_link = TRUE;
+                        break;
+                }
+
+        }
+        g_slist_free (tags);
+
+        g_object_get (page->images, "images", &images, NULL);
+        length = images->len;
+
+        gtk_widget_set_sensitive (page->add_recipe_button, !in_link);
+        gtk_widget_set_sensitive (page->link_image_button, !in_link && (length > 1));
+}
+
+static void
+cursor_moved (GObject    *object,
+              GParamSpec *pspec,
+              GrEditPage *page)
+{
+        update_link_button_sensitivity (page);
+}
+
+static void
+populate_recipe_list (GrEditPage *self)
+{
+        GtkTextBuffer *buffer;
+        GtkTextIter iter;
+
+        gtk_list_box_set_header_func (GTK_LIST_BOX (self->recipe_list),
+                                      all_headers, self, NULL);
+
+        gtk_list_box_set_filter_func (GTK_LIST_BOX (self->recipe_list),
+                                      recipe_filter_func, self, NULL);
+
+        g_signal_connect (self->recipe_list, "row-activated",
+                          G_CALLBACK (recipe_row_activated), self);
+
+        self->search = gr_recipe_search_new ();
+        g_signal_connect (self->search, "started", G_CALLBACK (search_started), self);
+        g_signal_connect (self->search, "hits-added", G_CALLBACK (search_hits_added), self);
+        g_signal_connect (self->search, "hits-removed", G_CALLBACK (search_hits_removed), self);
+        g_signal_connect (self->search, "finished", G_CALLBACK (search_finished), self);
+
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (self->instructions_field));
+        g_signal_connect (buffer, "notify::cursor-position", G_CALLBACK (cursor_moved), self);
+
+        gtk_text_buffer_get_start_iter (buffer, &iter);
+        gtk_text_buffer_create_mark (buffer, "saved_selection_bound", &iter, TRUE);
+}
+
+static void
+add_recipe_link (GtkButton *button, GrEditPage *page)
+{
+        GtkTextBuffer *buffer;
+        GtkTextIter start, end;
+
+        if (gtk_list_box_get_row_at_index (GTK_LIST_BOX (page->recipe_list), 0) == NULL) {
+                gr_recipe_search_stop (page->search);
+                gr_recipe_search_set_query (page->search, "na:");
+        }
+
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (page->instructions_field));
+
+        gtk_text_buffer_get_iter_at_mark (buffer, &start, gtk_text_buffer_get_selection_bound (buffer));
+        gtk_text_buffer_move_mark_by_name (buffer, "saved_selection_bound", &start);
+
+        if (gtk_text_buffer_get_selection_bounds (buffer, &start, &end)) {
+                g_autofree char *text = NULL;
+
+                text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+                gtk_entry_set_text (GTK_ENTRY (page->recipe_filter_entry), text);
+        }
+        else
+                gtk_entry_set_text (GTK_ENTRY (page->recipe_filter_entry), "");
+
+        gtk_popover_popup (GTK_POPOVER (page->recipe_popover));
+}
+
+static void
+recipe_reload (GrEditPage *page)
+{
+        container_remove_all (GTK_CONTAINER (page->recipe_list));
+}
+
+static void
+connect_store_signals (GrEditPage *page)
+{
+        GrRecipeStore *store;
+
+        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+
+        g_signal_connect_swapped (store, "recipe-added", G_CALLBACK (recipe_reload), page);
+        g_signal_connect_swapped (store, "recipe-removed", G_CALLBACK (recipe_reload), page);
+        g_signal_connect_swapped (store, "recipe-changed", G_CALLBACK (recipe_reload), page);
+}
+
+static void
+image_activated (GtkFlowBox *flowbox,
+                 GtkFlowBoxChild *child,
+                 GrEditPage *self)
+{
+        int idx;
+        GtkTextBuffer *buffer;
+        GtkTextIter start, end;
+        GtkTextTag *tag;
+        GdkRGBA color;
+        g_autofree char *url = NULL;
+        g_autofree char *text = NULL;
+
+        gdk_rgba_parse (&color, "blue");
+
+        idx = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (child), "image-idx"));
+
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (self->instructions_field));
+        tag = gtk_text_buffer_create_tag (buffer, NULL,
+                                          "foreground-rgba", &color,
+                                          "underline", PANGO_UNDERLINE_SINGLE,
+                                          NULL);
+        url = g_strdup_printf ("image:%d", idx);
+        g_object_set_data_full (G_OBJECT (tag), "href", g_strdup (url), g_free);
+
+        gtk_text_buffer_get_iter_at_mark (buffer, &start,
+                                          gtk_text_buffer_get_insert (buffer));
+        gtk_text_buffer_get_iter_at_mark (buffer, &end,
+                                          gtk_text_buffer_get_mark (buffer, "saved_selection_bound"));
+        text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+        gtk_text_buffer_delete (buffer, &start, &end);
+        gtk_text_buffer_insert (buffer, &start, "​", -1);
+
+        if (text[0] == '\0') {
+                char buf[6];
+                int len;
+
+                len = g_unichar_to_utf8 (0x2460 + idx, buf);
+                text = g_strndup (buf, len);
+        }
+
+        gtk_text_buffer_get_iter_at_mark (buffer, &start,
+                                          gtk_text_buffer_get_insert (buffer));
+        gtk_text_buffer_insert_with_tags (buffer, &start, text, -1, tag, NULL);
+
+        gtk_popover_popdown (GTK_POPOVER (self->image_popover));
+        gtk_widget_grab_focus (self->instructions_field);
+}
+
+static void
+add_image_link (GtkButton *button, GrEditPage *page)
+{
+        GtkTextBuffer *buffer;
+        GtkTextIter iter;
+
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (page->instructions_field));
+        gtk_text_buffer_get_iter_at_mark (buffer, &iter, gtk_text_buffer_get_selection_bound (buffer));
+        gtk_text_buffer_move_mark_by_name (buffer, "saved_selection_bound", &iter);
+
+        gtk_popover_popup (GTK_POPOVER (page->image_popover));
+}
+
 static void
 gr_edit_page_init (GrEditPage *page)
 {
-        GspellTextView *gspell_view;
-
         gtk_widget_set_has_window (GTK_WIDGET (page), FALSE);
         gtk_widget_init_template (GTK_WIDGET (page));
 
@@ -841,12 +1243,20 @@ gr_edit_page_init (GrEditPage *page)
         populate_season_combo (page);
         populate_ingredients_list (page);
         populate_units_list (page);
+        populate_recipe_list (page);
+        connect_store_signals (page);
 
-        gspell_view = gspell_text_view_get_from_gtk_text_view (GTK_TEXT_VIEW (page->description_field));
-        gspell_text_view_basic_setup (gspell_view);
+#ifdef ENABLE_GSPELL
+        {
+                GspellTextView *gspell_view;
 
-        gspell_view = gspell_text_view_get_from_gtk_text_view (GTK_TEXT_VIEW (page->instructions_field));
-        gspell_text_view_basic_setup (gspell_view);
+                gspell_view = gspell_text_view_get_from_gtk_text_view (GTK_TEXT_VIEW (page->description_field));
+                gspell_text_view_basic_setup (gspell_view);
+
+                gspell_view = gspell_text_view_get_from_gtk_text_view (GTK_TEXT_VIEW (page->instructions_field));
+                gspell_text_view_basic_setup (gspell_view);
+        }
+#endif
 }
 
 static gboolean
@@ -934,6 +1344,13 @@ gr_edit_page_class_init (GrEditPageClass *klass)
         gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, amount_search_button);
         gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, amount_search_button_label);
         gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, amount_search_revealer);
+        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, recipe_popover);
+        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, recipe_list);
+        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, recipe_filter_entry);
+        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, add_recipe_button);
+        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, link_image_button);
+        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, image_popover);
+        gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (klass), GrEditPage, image_flowbox);
 
         gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), dismiss_error);
         gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (klass), add_image);
@@ -955,6 +1372,13 @@ gr_edit_page_class_init (GrEditPageClass *klass)
         gtk_widget_class_bind_template_callback (widget_class, unit_filter_activated);
         gtk_widget_class_bind_template_callback (widget_class, amount_search_button_clicked);
         gtk_widget_class_bind_template_callback (widget_class, popover_keypress_handler);
+        gtk_widget_class_bind_template_callback (widget_class, add_recipe_link);
+        gtk_widget_class_bind_template_callback (widget_class, add_image_link);
+        gtk_widget_class_bind_template_callback (widget_class, image_activated);
+
+        gtk_widget_class_bind_template_callback (widget_class, recipe_filter_changed);
+        gtk_widget_class_bind_template_callback (widget_class, recipe_filter_stop);
+        gtk_widget_class_bind_template_callback (widget_class, recipe_filter_activated);
 }
 
 GtkWidget *
@@ -1062,7 +1486,6 @@ add_ingredients_segment (GrEditPage *page,
         GtkWidget *button;
         GtkWidget *stack;
         GtkWidget *image;
-        GspellEntry *gspell_entry;
 
         segment = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
         gtk_widget_set_margin_top (segment, 20);
@@ -1090,10 +1513,17 @@ add_ingredients_segment (GrEditPage *page,
         g_object_set_data (G_OBJECT (segment), "entry", entry);
         gtk_widget_set_halign (box, GTK_ALIGN_FILL);
         gtk_widget_show (entry);
-        gtk_entry_set_text (GTK_ENTRY (entry), segment_label[0] ? segment_label : _("Ingredients for …"));
+        gtk_entry_set_placeholder_text (GTK_ENTRY (entry), _("Name of the List"));
+        gtk_entry_set_text (GTK_ENTRY (entry), segment_label[0] ? segment_label : "");
 
-        gspell_entry = gspell_entry_get_from_gtk_entry (GTK_ENTRY (entry));
-        gspell_entry_basic_setup (gspell_entry);
+#if defined(ENABLE_GSPELL) && defined(GSPELL_TYPE_ENTRY)
+        {
+                GspellEntry *gspell_entry;
+
+                gspell_entry = gspell_entry_get_from_gtk_entry (GTK_ENTRY (entry));
+                gspell_entry_basic_setup (gspell_entry);
+        }
+#endif
 
         gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
 
@@ -1277,6 +1707,80 @@ gr_edit_page_clear (GrEditPage *page)
         g_clear_object (&page->recipe);
 }
 
+static void
+set_instructions (GtkTextView *text_view,
+                  const char  *text)
+{
+        GtkTextBuffer *buffer;
+        GtkTextIter iter;
+        GtkTextTag *tag;
+        const char *p;
+        const char *p1, *p2, *q1, *q2, *r1, *r2;
+        GdkRGBA color;
+
+        gdk_rgba_parse (&color, "blue");
+        buffer = gtk_text_view_get_buffer (text_view);
+        gtk_text_buffer_set_text (buffer, "", -1);
+
+        p = text;
+        while (*p) {
+                g_autofree char *recipe_id = NULL;
+                g_autofree char *url = NULL;
+                int image_idx;
+
+                q1 = NULL;
+                r1 = NULL;
+                p1 = strstr (p, "<a href=\"");
+                if (!p1)
+                        break;
+                p2 = p1 + strlen ("<a href=\"");
+                if (strncmp (p2, "recipe:", strlen ("recipe:")) == 0) {
+                        p2 = p2 + strlen ("recipe:");
+                        q1 = strstr (p2, "\">");
+                        recipe_id = g_strndup (p2, q1 - p2);
+                        url = g_strconcat ("recipe:", recipe_id, NULL);
+                }
+                else if (strncmp (p2, "image:", strlen ("image:")) == 0) {
+                        p2 = p2 + strlen ("image:");
+                        q1 = strstr (p2, "\">");
+                        image_idx = (int)g_ascii_strtoll (p2, NULL, 10);
+                        url = g_strdup_printf ("image:%d", image_idx);
+                }
+                else {
+                        p = p2;
+                        continue;
+                }
+
+                if (!q1)
+                        break;
+
+                q2 = q1 + strlen ("\">");
+                r1 = strstr (q2, "</a>");
+
+                if (!r1)
+                        break;
+
+                r2 = r1 + strlen ("</a>");
+
+                gtk_text_buffer_get_end_iter (buffer, &iter);
+                gtk_text_buffer_insert (buffer, &iter, p, p1 - p);
+
+                tag = gtk_text_buffer_create_tag (buffer, NULL,
+                                                  "foreground-rgba", &color,
+                                                  "underline", PANGO_UNDERLINE_SINGLE,
+                                                  NULL);
+                g_object_set_data_full (G_OBJECT (tag), "href", g_strdup (url), g_free);
+
+                gtk_text_buffer_get_end_iter (buffer, &iter);
+                gtk_text_buffer_insert_with_tags (buffer, &iter, q2, r1 - q2, tag, NULL);
+
+                p = r2;
+        }
+
+        gtk_text_buffer_get_end_iter (buffer, &iter);
+        gtk_text_buffer_insert (buffer, &iter, p, -1);
+}
+
 void
 gr_edit_page_edit (GrEditPage *page,
                    GrRecipe   *recipe)
@@ -1293,8 +1797,6 @@ gr_edit_page_edit (GrEditPage *page,
         const char *description;
         const char *instructions;
         const char *ingredients;
-        g_autofree char *image_path = NULL;
-        g_autoptr(GdkPixbuf) pixbuf = NULL;
         GrDiets diets;
         g_autoptr(GArray) images = NULL;
         g_autoptr(GrChef) chef = NULL;
@@ -1331,7 +1833,7 @@ gr_edit_page_edit (GrEditPage *page,
         set_spiciness (page, spiciness);
         gtk_spin_button_set_value (GTK_SPIN_BUTTON (page->serves_spin), serves);
         set_text_view_text (GTK_TEXT_VIEW (page->description_field), description);
-        set_text_view_text (GTK_TEXT_VIEW (page->instructions_field), instructions);
+        set_instructions (GTK_TEXT_VIEW (page->instructions_field), instructions);
 
         populate_ingredients (page, ingredients);
 
@@ -1374,11 +1876,9 @@ account_response (GDBusConnection *connection,
         g_variant_get (parameters, "(u@a{sv})", &response, &options);
 
         if (response == 0) {
-                g_autoptr(GdkPixbuf) pixbuf = NULL;
                 const char *id;
                 const char *name;
                 const char *uri;
-                g_autofree char *path = NULL;
                 GrRecipeStore *store;
 
                 store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
@@ -1565,18 +2065,58 @@ ensure_user_chef (GrRecipeStore *store,
 }
 
 static char *
-generate_id (const char *name,
-             const char *author)
+get_instructions (GtkTextView *text_view)
 {
-        char *s, *q;
+        GtkTextBuffer *buffer;
+        GtkTextIter start, end;
+        GString *s;
+        g_autofree char *last_text = NULL;
 
-        s = g_strconcat (name, "_by_", author, NULL);
-        for (q = s; *q; q = g_utf8_find_next_char (q, NULL)) {
-                if (*q == ']' || *q == '[' || g_ascii_iscntrl (*q))
-                        *q = '_';
+        s = g_string_new ("");
+
+        buffer = gtk_text_view_get_buffer (text_view);
+        gtk_text_buffer_get_start_iter (buffer, &start);
+        end = start;
+        while (gtk_text_iter_forward_to_tag_toggle (&end, NULL)) {
+                g_autofree char *text = NULL;
+                GSList *tags, *l;
+                GtkTextTag *tag;
+
+                text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+                g_string_append (s, text);
+                tags = gtk_text_iter_get_tags (&end);
+                tag = NULL;
+                for (l = tags; l; l = l->next) {
+                        if (g_object_get_data (G_OBJECT (l->data), "href")) {
+                                tag = l->data;
+                                break;
+                        }
+                }
+                g_slist_free (tags);
+
+                if (tag) {
+                        g_autofree char *name = NULL;
+                        const char *url;
+
+                        start = end;
+
+                        gtk_text_iter_forward_to_tag_toggle (&end, tag);
+                        name = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+
+                        url = (const char*)g_object_get_data (G_OBJECT (tag), "href");
+
+                        g_string_append_printf (s, "<a href=\"%s\">", url);
+                        g_string_append (s, name);
+                        g_string_append (s, "</a>");
+                }
+                start = end;
         }
 
-        return s;
+        gtk_text_buffer_get_end_iter (buffer, &end);
+        last_text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+        g_string_append (s, last_text);
+
+        return g_string_free (s, FALSE);
 }
 
 gboolean
@@ -1611,7 +2151,7 @@ gr_edit_page_save (GrEditPage *page)
         serves = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (page->serves_spin));
         ingredients = collect_ingredients (page);
         description = get_text_view_text (GTK_TEXT_VIEW (page->description_field));
-        instructions = get_text_view_text (GTK_TEXT_VIEW (page->instructions_field));
+        instructions = get_instructions (GTK_TEXT_VIEW (page->instructions_field));
         diets = (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->gluten_free_check)) ? GR_DIET_GLUTEN_FREE : 0) |
                 (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->nut_free_check)) ? GR_DIET_NUT_FREE : 0) |
                 (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->vegan_check)) ? GR_DIET_VEGAN : 0) |
@@ -1626,7 +2166,7 @@ gr_edit_page_save (GrEditPage *page)
                 const char *author;
 
                 author = gr_recipe_get_author (page->recipe);
-                id = generate_id (name, author);
+                id = generate_id ("R_", name, "_by_", author, NULL);
                 old_id = g_strdup (gr_recipe_get_id (page->recipe));
                 g_object_set (page->recipe,
                               "id", id,
@@ -1653,7 +2193,7 @@ gr_edit_page_save (GrEditPage *page)
 
                 author = gr_recipe_store_get_user_key (store);
                 ensure_user_chef (store, page);
-                id = generate_id (name, author);
+                id = generate_id ("R_", name, "_by_", author, NULL);
                 recipe = g_object_new (GR_TYPE_RECIPE,
                                        "id", id,
                                        "name", name,
@@ -1672,13 +2212,12 @@ gr_edit_page_save (GrEditPage *page)
                                        "images", images,
                                        NULL);
                 ret = gr_recipe_store_add_recipe (store, recipe, &error);
+                g_set_object (&page->recipe, recipe);
         }
 
         if (!ret)
                 goto error;
 
-        g_clear_object (&page->recipe);
-
         return TRUE;
 
 error:
@@ -1689,3 +2228,10 @@ error:
 
         return FALSE;
 }
+
+GrRecipe *
+gr_edit_page_get_recipe (GrEditPage *page)
+{
+        return page->recipe;
+}
+
