@@ -52,6 +52,7 @@ ingredient_free (Ingredient *ing)
 {
         g_free (ing->name);
         g_free (ing->unit);
+        g_free (ing->segment);
         g_free (ing);
 }
 
@@ -130,9 +131,9 @@ gr_ingredients_list_populate (GrIngredientsList  *ingredients,
                         segment = NULL;
 
                 if (segment)
-                        g_hash_table_add (ingredients->segments, g_strdup (segment));
+                        g_hash_table_add (ingredients->segments, g_strdup (_(segment)));
                 if (!gr_ingredients_list_add_one (ingredients, lines[i],
-                                                  (char *)(segment ? segment : ""), error)) {
+                                                  (char *)(segment ? _(segment) : ""), error)) {
                         g_message ("Failed to parse ingredients line '%s'", lines[i]);
                 }
         }
@@ -161,6 +162,7 @@ gr_ingredients_list_populate_new_format (GrIngredientsList  *ingredients,
                 char *ingredient;
                 char *segment;
                 const char *u;
+                const char *s;
                 Ingredient *ing;
                 g_autoptr(GError) local_error = NULL;
 
@@ -173,7 +175,7 @@ gr_ingredients_list_populate_new_format (GrIngredientsList  *ingredients,
                 amount = fields[0];
                 unit = fields[1];
                 ingredient = fields[2];
-                segment = fields[3];
+                segment = (char *)(fields[3][0] ? _(fields[3]) : "");
 
                 ing = g_new0 (Ingredient, 1);
                 gr_number_set_fraction (&ing->amount, 0, 0);
@@ -187,15 +189,20 @@ gr_ingredients_list_populate_new_format (GrIngredientsList  *ingredients,
                 u = "";
                 if (unit[0] != '\0' &&
                     ((u = gr_unit_parse (&unit, &local_error)) == NULL)) {
-                        g_message ("failed to unit amount in '%s': %s", unit, local_error->message);
+                        g_message ("failed to parse unit in '%s': %s", unit, local_error->message);
                         g_free (ing);
                         continue;
                 }
 
                 ing->unit = g_strdup (u);
-                ing->name = g_strdup (ingredient);
                 ing->segment = g_strdup (segment);
 
+                s = gr_ingredient_find (ingredient);
+                if (s)
+                        ing->name = g_strdup (s);
+                else
+                        ing->name = g_strdup (ingredient);
+
                 g_hash_table_add (ingredients->segments, g_strdup (segment));
                 ingredients->ingredients = g_list_append (ingredients->ingredients, ing);
         }
