@@ -22,8 +22,10 @@
 #include "config.h"
 
 #include <glib/gi18n.h>
+#ifdef ENABLE_AUTOAR
 #include <gnome-autoar/gnome-autoar.h>
 #include "glnx-shutil.h"
+#endif
 
 #include "gr-recipe-exporter.h"
 #include "gr-images.h"
@@ -41,7 +43,9 @@ struct _GrRecipeExporter
         GrRecipe *recipe;
         GtkWindow *window;
 
+#ifdef ENABLE_AUTOAR
         AutoarCompressor *compressor;
+#endif
         GFile *dest;
         GFile *output;
         GList *sources;
@@ -92,20 +96,24 @@ gr_recipe_exporter_new (GtkWindow *parent)
 static void
 cleanup_export (GrRecipeExporter *exporter)
 {
+#ifdef ENABLE_AUTOAR
         g_autoptr(GError) error = NULL;
 
         if (!glnx_shutil_rm_rf_at (-1, exporter->dir, NULL, &error))
                 g_warning ("Failed to clean up temp directory %s: %s", exporter->dir, error->message);
 
+        g_clear_object (&exporter->compressor);
+#endif
+
         g_clear_pointer (&exporter->dir, g_free);
         g_clear_object (&exporter->recipe);
-        g_clear_object (&exporter->compressor);
         g_clear_object (&exporter->output);
         g_clear_object (&exporter->dest);
         g_list_free_full (exporter->sources, g_object_unref);
         exporter->sources = NULL;
 }
 
+#ifdef ENABLE_AUTOAR
 static void
 completed_cb (AutoarCompressor *compressor,
               GrRecipeExporter *exporter)
@@ -129,26 +137,6 @@ completed_cb (AutoarCompressor *compressor,
         cleanup_export (exporter);
 }
 
-static void
-error_cb (AutoarCompressor *compressor,
-          GError           *error,
-          GrRecipeExporter *exporter)
-{
-        GtkWidget *dialog;
-
-        dialog = gtk_message_dialog_new (exporter->window,
-                                         GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
-                                         GTK_MESSAGE_ERROR,
-                                         GTK_BUTTONS_OK,
-                                         _("Error while exporting recipe %s:\n%s"),
-                                         gr_recipe_get_name (exporter->recipe),
-                                         error->message);
-        g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
-        gtk_widget_show (dialog);
-
-        cleanup_export (exporter);
-}
-
 static void
 decide_dest_cb (AutoarCompressor *compressor,
                 GFile            *file,
@@ -156,11 +144,18 @@ decide_dest_cb (AutoarCompressor *compressor,
 {
         g_set_object (&exporter->dest, file);
 }
+#endif
 
 static gboolean
 prepare_export (GrRecipeExporter  *exporter,
                 GError           **error)
 {
+#ifndef ENABLE_AUTOAR
+        g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                     _("This build does not support exporting"));
+
+        return FALSE;
+#else
         g_autofree char *path = NULL;
         g_autoptr(GKeyFile) keyfile = NULL;
         const char *key;
@@ -196,7 +191,8 @@ prepare_export (GrRecipeExporter  *exporter,
         path = g_build_filename (exporter->dir, "recipes.db", NULL);
         keyfile = g_key_file_new ();
 
-        key = name = gr_recipe_get_name (exporter->recipe);
+        key = gr_recipe_get_id (exporter->recipe);
+        name = gr_recipe_get_name (exporter->recipe);
         author = gr_recipe_get_author (exporter->recipe);
         description = gr_recipe_get_description (exporter->recipe);
         serves = gr_recipe_get_serves (exporter->recipe);
@@ -322,8 +318,28 @@ prepare_export (GrRecipeExporter  *exporter,
         }
 
         exporter->sources = g_list_append (exporter->sources, g_file_new_for_path (path));
-
         return TRUE;
+#endif
+}
+
+static void
+error_cb (gpointer          compressor,
+          GError           *error,
+          GrRecipeExporter *exporter)
+{
+        GtkWidget *dialog;
+
+        dialog = gtk_message_dialog_new (exporter->window,
+                                         GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
+                                         GTK_MESSAGE_ERROR,
+                                         GTK_BUTTONS_OK,
+                                         _("Error while exporting recipe %s:\n%s"),
+                                         gr_recipe_get_name (exporter->recipe),
+                                         error->message);
+        g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+        gtk_widget_show (dialog);
+
+        cleanup_export (exporter);
 }
 
 void
@@ -331,7 +347,6 @@ gr_recipe_exporter_export_to (GrRecipeExporter *exporter,
                               GrRecipe         *recipe,
                               GFile            *file)
 {
-        g_autoptr(GFile) output = NULL;
         g_autoptr(GError) error = NULL;
 
         g_set_object (&exporter->recipe, recipe);
@@ -342,11 +357,14 @@ gr_recipe_exporter_export_to (GrRecipeExporter *exporter,
                 return;
         }
 
+#ifdef ENABLE_AUTOAR
         exporter->compressor = autoar_compressor_new (exporter->sources, exporter->output, AUTOAR_FORMAT_TAR, AUTOAR_FILTER_GZIP, FALSE);
 
+        autoar_compressor_set_output_is_dest (exporter->compressor, TRUE);
         g_signal_connect (exporter->compressor, "decide-dest", G_CALLBACK (decide_dest_cb), exporter);
         g_signal_connect (exporter->compressor, "completed", G_CALLBACK (completed_cb), exporter);
         g_signal_connect (exporter->compressor, "error", G_CALLBACK (error_cb), exporter);
 
         autoar_compressor_start_async (exporter->compressor, NULL);
+#endif
 }
