@@ -109,7 +109,7 @@ begin_print (GtkPrintOperation *operation,
         pango_layout_set_font_description (printer->title_layout, title_font);
 
         s = g_string_new ("");
-        g_string_append (s, gr_recipe_get_name (printer->recipe));
+        g_string_append (s, gr_recipe_get_translated_name (printer->recipe));
         g_string_append (s, "\n\n");
 
         pango_layout_set_text (printer->title_layout, s->str, s->len);
@@ -140,7 +140,7 @@ begin_print (GtkPrintOperation *operation,
         pango_layout_set_tabs (printer->bottom_layout, tabs);
         pango_tab_array_free (tabs);
 
-        g_string_append (s, gr_recipe_get_description (printer->recipe));
+        g_string_append (s, gr_recipe_get_translated_description (printer->recipe));
         g_string_append (s, "\n\n");
 
         attrs = pango_attr_list_new ();
@@ -205,13 +205,13 @@ begin_print (GtkPrintOperation *operation,
         attr = pango_attr_font_desc_new (title_font);
         attr->start_index = s->len;
 
-        g_string_append (s, _("Instructions"));
+        g_string_append (s, _("Directions"));
 
         attr->end_index = s->len + 1;
         pango_attr_list_insert (attrs, attr);
 
         g_string_append (s, "\n\n");
-        g_string_append (s, gr_recipe_get_instructions (printer->recipe));
+        g_string_append (s, gr_recipe_get_translated_instructions (printer->recipe));
 
         pango_layout_set_text (printer->bottom_layout, s->str, s->len);
         pango_layout_set_attributes (printer->bottom_layout, attrs);
@@ -279,8 +279,6 @@ draw_page (GtkPrintOperation *operation,
         PangoRectangle logical_rect;
         int baseline;
         int width;
-        g_autoptr(GArray) images = NULL;
-        g_autoptr(GdkPixbuf) pixbuf = NULL;
         int start, end, i;
         PangoLayoutIter *iter;
         double start_pos;
