@@ -21,7 +21,6 @@
 #include "config.h"
 
 #include <stdlib.h>
-#include <sys/statfs.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
@@ -1238,7 +1237,6 @@ gr_recipe_store_get_all_ingredients (GrRecipeStore *self,
         while (g_hash_table_iter_next (&iter, NULL, (gpointer *)&recipe)) {
                 const char *ingredients;
                 GrIngredientsList *list = NULL;
-                g_autofree char **ret = NULL;
                 g_autofree char **segments = NULL;
 
                 ingredients = gr_recipe_get_ingredients (recipe);
@@ -1249,6 +1247,7 @@ gr_recipe_store_get_all_ingredients (GrRecipeStore *self,
                 list = gr_ingredients_list_new (ingredients);
                 segments = gr_ingredients_list_get_segments (list);
                 for (j = 0; segments[j]; j++) {
+                        g_autofree char **ret = NULL;
                         ret = gr_ingredients_list_get_ingredients (list, segments[j]);
                         for (i = 0; ret[i]; i++)
                                 g_hash_table_add (ingreds, ret[i]);
@@ -1414,7 +1413,7 @@ struct _GrRecipeSearch
 
         GrRecipeStore *store;
 
-        char *query;
+        char **query;
 
         gulong idle;
         GHashTableIter iter;
@@ -1469,8 +1468,10 @@ clear_pending (GrRecipeSearch *search)
 static void
 send_pending (GrRecipeSearch *search)
 {
-        g_signal_emit (search, search_signals[HITS_ADDED], 0, search->pending);
-        clear_pending (search);
+        if (search->n_pending > 0) {
+                g_signal_emit (search, search_signals[HITS_ADDED], 0, search->pending);
+                clear_pending (search);
+        }
 }
 
 static void
@@ -1480,33 +1481,42 @@ clear_results (GrRecipeSearch *search)
         search->results = NULL;
 }
 
+static gboolean
+recipe_matches (GrRecipeSearch *search,
+                GrRecipe       *recipe)
+{
+        if (strcmp (search->query[0], "is:favorite") == 0)
+                return gr_recipe_store_is_favorite (search->store, recipe);
+        else
+                return gr_recipe_matches (recipe, (const char **)search->query);
+}
+
 static gboolean
 search_idle (gpointer data)
 {
         GrRecipeSearch *search = data;
-        int count;
         const char *key;
         GrRecipe *recipe;
+        gint64 start_time;
+
+        start_time = g_get_monotonic_time ();
 
-        count = 0;
         while (g_hash_table_iter_next (&search->iter, (gpointer *)&key, (gpointer *)&recipe)) {
-                count++;
-
-                if (gr_recipe_matches (recipe, search->query))
+                if (recipe_matches (search, recipe))
                         add_pending (search, recipe);
 
                 if (search->n_pending > 2) {
                         send_pending (search);
-                        return G_SOURCE_CONTINUE;
                 }
 
-                if (count == 9) {
-                        return G_SOURCE_CONTINUE;
+                if (g_get_monotonic_time () >= start_time + 4000) {
+                        send_pending (search);
+                        search->idle = g_timeout_add (16, search_idle, search);
+                        return G_SOURCE_REMOVE;
                 }
         }
 
-        if (search->n_pending > 0)
-                send_pending (search);
+        send_pending (search);
 
         g_signal_emit (search, search_signals[FINISHED], 0);
         search->idle = 0;
@@ -1526,8 +1536,8 @@ start_search (GrRecipeSearch *search)
                 g_hash_table_iter_init (&search->iter, search->store->recipes);
                 clear_pending (search);
                 clear_results (search);
-                search->idle = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, search_idle, search, NULL);
                 g_signal_emit (search, search_signals[STARTED], 0);
+                search_idle (search);
         }
 }
 
@@ -1555,7 +1565,7 @@ refilter_existing_results (GrRecipeSearch *search)
         for (l = search->results; l; l = l->next) {
                 GrRecipe *recipe = l->data;
 
-                if (gr_recipe_matches (recipe, search->query))
+                if (recipe_matches (search, recipe))
                         accepted = g_list_append (accepted, recipe);
                 else
                         rejected = g_list_append (rejected, recipe);
@@ -1574,36 +1584,78 @@ refilter_existing_results (GrRecipeSearch *search)
 }
 
 static gboolean
-query_is_narrowing (GrRecipeSearch *search,
-                    const char     *query)
+query_is_narrowing (GrRecipeSearch  *search,
+                    const char     **query)
 {
-        /* FIXME: can be more precise */
-        return search->query && g_str_has_prefix (query, search->query);
+        int i, j;
+
+        /* Being narrower means having more conditions */
+        if (search->query == NULL)
+                return FALSE;
+
+        for (i = 0; search->query[i]; i++) {
+                const char *term = search->query[i];
+
+                if (g_str_has_prefix (term, "i+:") ||
+                    g_str_has_prefix (term, "i-:") ||
+                    g_str_has_prefix (term, "by:") ||
+                    g_str_has_prefix (term, "se:") ||
+                    g_str_has_prefix (term, "me:") ||
+                    g_str_has_prefix (term, "di:")) {
+                        if (!g_strv_contains (query, term))
+                                return FALSE;
+                }
+                else {
+                        gboolean res = FALSE;
+                        for (j = 0; query[j]; j++) {
+                                if (g_str_has_prefix (query[j], term)) {
+                                        res = TRUE;
+                                        break;
+                                }
+                        }
+                        if (!res)
+                                return FALSE;
+                }
+        }
+
+        return TRUE;
 }
 
 void
 gr_recipe_search_stop (GrRecipeSearch *search)
 {
         stop_search (search);
-        g_clear_pointer (&search->query, g_free);
+        g_clear_pointer (&search->query, g_strfreev);
 }
 
 void
 gr_recipe_search_set_query (GrRecipeSearch *search,
                             const char     *query)
+{
+        g_auto(GStrv) strv = NULL;
+
+        if (query)
+                strv = g_strsplit (query, " ", -1);
+
+        gr_recipe_search_set_terms (search, (const char **)strv);
+}
+
+void
+gr_recipe_search_set_terms (GrRecipeSearch  *search,
+                            const char     **terms)
 {
         gboolean narrowing;
 
-        if (query == NULL) {
+        if (terms == NULL || terms[0] == NULL) {
                 stop_search (search);
-                g_clear_pointer (&search->query, g_free);
+                g_clear_pointer (&search->query, g_strfreev);
                 return;
         }
 
-        narrowing = query_is_narrowing (search, query);
+        narrowing = query_is_narrowing (search, terms);
 
-        g_free (search->query);
-        search->query = g_strdup (query);
+        g_strfreev (search->query);
+        search->query = g_strdupv ((char **)terms);
 
         if (narrowing) {
                 refilter_existing_results (search);
@@ -1614,10 +1666,10 @@ gr_recipe_search_set_query (GrRecipeSearch *search,
         }
 }
 
-const char *
-gr_recipe_search_get_query (GrRecipeSearch *search)
+const char **
+gr_recipe_search_get_terms (GrRecipeSearch *search)
 {
-        return search->query;
+        return (const char **)search->query;
 }
 
 static void
@@ -1626,7 +1678,7 @@ gr_recipe_search_finalize (GObject *object)
         GrRecipeSearch *search = (GrRecipeSearch *)object;
 
         stop_search (search);
-        g_free (search->query);
+        g_strfreev (search->query);
         g_object_unref (search->store);
 
         G_OBJECT_CLASS (gr_recipe_search_parent_class)->finalize (object);
