@@ -26,6 +26,7 @@
 
 #include "gr-recipe.h"
 #include "gr-images.h"
+#include "gr-utils.h"
 #include "types.h"
 
 struct _GrRecipe
@@ -59,6 +60,10 @@ struct _GrRecipe
         int spiciness;
 
         gboolean readonly;
+
+        char *translated_name;
+        char *translated_description;
+        char *translated_instructions;
 };
 
 G_DEFINE_TYPE (GrRecipe, gr_recipe, G_TYPE_OBJECT)
@@ -112,6 +117,10 @@ gr_recipe_finalize (GObject *object)
         g_date_time_unref (self->mtime);
         g_date_time_unref (self->ctime);
 
+        g_free (self->translated_name);
+        g_free (self->translated_description);
+        g_free (self->translated_instructions);
+
         G_OBJECT_CLASS (gr_recipe_parent_class)->finalize (object);
 }
 
@@ -259,7 +268,8 @@ gr_recipe_set_property (GObject      *object,
                 g_free (self->name);
                 g_free (self->cf_name);
                 self->name = g_value_dup_string (value);
-                self->cf_name = g_utf8_casefold (self->name, -1);
+                self->translated_name = translate_multiline_string (self->name);
+                self->cf_name = g_utf8_casefold (self->translated_name, -1);
                 update_mtime (self);
                 break;
 
@@ -267,7 +277,8 @@ gr_recipe_set_property (GObject      *object,
                 g_free (self->description);
                 g_free (self->cf_description);
                 self->description = g_value_dup_string (value);
-                self->cf_description = g_utf8_casefold (self->description, -1);
+                self->translated_description = translate_multiline_string (self->description);
+                self->cf_description = g_utf8_casefold (self->translated_description, -1);
                 update_mtime (self);
                 break;
 
@@ -334,6 +345,7 @@ gr_recipe_set_property (GObject      *object,
         case PROP_INSTRUCTIONS:
                 g_free (self->instructions);
                 self->instructions = g_value_dup_string (value);
+                self->translated_instructions = translate_multiline_string (self->instructions);
                 update_mtime (self);
                 break;
 
@@ -502,6 +514,12 @@ gr_recipe_get_name (GrRecipe *recipe)
         return recipe->name;
 }
 
+const char *
+gr_recipe_get_translated_name (GrRecipe *recipe)
+{
+        return recipe->translated_name;
+}
+
 const char *
 gr_recipe_get_author (GrRecipe *recipe)
 {
@@ -514,6 +532,12 @@ gr_recipe_get_description (GrRecipe *recipe)
         return recipe->description;
 }
 
+const char *
+gr_recipe_get_translated_description (GrRecipe *recipe)
+{
+        return recipe->translated_description;
+}
+
 int
 gr_recipe_get_serves (GrRecipe *recipe)
 {
@@ -568,6 +592,12 @@ gr_recipe_get_instructions (GrRecipe *recipe)
         return recipe->instructions;
 }
 
+const char *
+gr_recipe_get_translated_instructions (GrRecipe *recipe)
+{
+        return recipe->translated_instructions;
+}
+
 const char *
 gr_recipe_get_notes (GrRecipe *recipe)
 {
@@ -580,6 +610,18 @@ gr_recipe_contains_garlic (GrRecipe *recipe)
         return recipe->garlic;
 }
 
+gboolean
+gr_recipe_contains_dairy (GrRecipe *recipe)
+{
+        return (recipe->diets & GR_DIET_MILK_FREE) == 0;
+}
+
+gboolean
+gr_recipe_contains_gluten (GrRecipe *recipe)
+{
+        return (recipe->diets & GR_DIET_GLUTEN_FREE) == 0;
+}
+
 int
 gr_recipe_get_spiciness (GrRecipe *recipe)
 {
@@ -604,31 +646,34 @@ gr_recipe_is_readonly (GrRecipe *recipe)
         return recipe->readonly;
 }
 
-/* term is assumed to be g_utf8_casefold'ed */
+/* terms are assumed to be g_utf8_casefold'ed where appropriate */
 gboolean
-gr_recipe_matches (GrRecipe   *recipe,
-                   const char *term)
+gr_recipe_matches (GrRecipe    *recipe,
+                   const char **terms)
 {
-        g_auto(GStrv) terms = NULL;
         int i;
 
-        terms = g_strsplit (term, " ", -1);
-
         for (i = 0; terms[i]; i++) {
                 if (g_str_has_prefix (terms[i], "i+:")) {
-                        if (!recipe->cf_ingredients || strstr (recipe->cf_ingredients, terms[i] + 3) == NULL) {
+                        if (!recipe->ingredients || strstr (recipe->ingredients, terms[i] + 3) == NULL) {
                                 return FALSE;
                         }
                         continue;
                 }
                 else if (g_str_has_prefix (terms[i], "i-:")) {
-                        if (recipe->cf_ingredients && strstr (recipe->cf_ingredients, terms[i] + 3) != NULL) {
+                        if (recipe->ingredients && strstr (recipe->ingredients, terms[i] + 3) != NULL) {
                                 return FALSE;
                         }
                         continue;
                 }
                 else if (g_str_has_prefix (terms[i], "by:")) {
-                        if (!recipe->author || strstr (recipe->author, terms[i] + 3) == NULL) {
+                        if (!recipe->author || strcmp (recipe->author, terms[i] + 3) != 0) {
+                                return FALSE;
+                        }
+                        continue;
+                }
+                else if (g_str_has_prefix (terms[i], "se:")) {
+                        if (!recipe->season || strcmp (recipe->season , terms[i] + 3) != 0) {
                                 return FALSE;
                         }
                         continue;
@@ -662,6 +707,12 @@ gr_recipe_matches (GrRecipe   *recipe,
 
                         continue;
                 }
+                else if (g_str_has_prefix (terms[i], "na:")) {
+                        if (recipe->cf_name && strstr (recipe->cf_name, terms[i] + 3) == NULL)
+                                return FALSE;
+                        continue;
+
+                }
 
                 if (recipe->cf_name && strstr (recipe->cf_name, terms[i]) != NULL)
                         continue;
