@@ -48,5 +48,10 @@ GDateTime * date_time_from_string (const char *string);
 gboolean skip_whitespace (char **input);
 gboolean space_or_nul    (char p);
 
+char *translate_multiline_string (const char *s);
 
-G_END_DECLS
+char *generate_id (const char *s, ...);
+
+void start_recording (void);
+void stop_recording (void);
+void record_step (const char *blurb);
