@@ -67,12 +67,12 @@ G_DEFINE_TYPE (GrWindow, gr_window, GTK_TYPE_APPLICATION_WINDOW)
 
 typedef struct
 {
-        gchar *page;
-        gchar *header_start_child;
-        gchar *header_title_child;
-        gchar *header_end_child;
-        gchar *header_title;
-        gchar *search;
+        char *page;
+        char *header_start_child;
+        char *header_title_child;
+        char *header_end_child;
+        char *header_title;
+        char **search;
 } BackEntry;
 
 static void
@@ -83,7 +83,7 @@ back_entry_free (BackEntry *entry)
         g_free (entry->header_title_child);
         g_free (entry->header_end_child);
         g_free (entry->header_title);
-        g_free (entry->search);
+        g_strfreev (entry->search);
         g_free (entry);
 }
 
@@ -91,16 +91,21 @@ static void
 save_back_entry (GrWindow *window)
 {
         BackEntry *entry;
+        const char *page;
+
+        page = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
+        if (strcmp (page, "details") == 0 || strcmp (page, "edit") == 0)
+                return;
 
         entry = g_new (BackEntry, 1);
-        entry->page = g_strdup (gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack)));
+        entry->page = g_strdup (page);
         entry->header_start_child = g_strdup (gtk_stack_get_visible_child_name (GTK_STACK (window->header_start_stack)));
         entry->header_title_child = g_strdup (gtk_stack_get_visible_child_name (GTK_STACK (window->header_title_stack)));
         entry->header_end_child = g_strdup (gtk_stack_get_visible_child_name (GTK_STACK (window->header_end_stack)));
         entry->header_title = g_strdup (gtk_header_bar_get_title (GTK_HEADER_BAR (window->header)));
 
         if (strcmp (entry->page, "search") == 0)
-                entry->search = g_strdup (gr_query_editor_get_query (GR_QUERY_EDITOR (window->search_bar)));
+                entry->search = g_strdupv ((char **)gr_query_editor_get_terms (GR_QUERY_EDITOR (window->search_bar)));
         else
                 entry->search = NULL;
 
@@ -126,7 +131,7 @@ go_back (GrWindow *window)
 
         if (strcmp (entry->page, "search") == 0) {
                 gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (window->search_bar), TRUE);
-                gr_query_editor_set_query (GR_QUERY_EDITOR (window->search_bar), entry->search);
+                gr_query_editor_set_terms (GR_QUERY_EDITOR (window->search_bar), (const char **)entry->search);
         }
         else {
                 gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (window->search_bar), FALSE);
@@ -154,8 +159,13 @@ new_recipe (GrWindow *window)
 static void
 add_recipe (GrWindow *window)
 {
-        if (gr_edit_page_save (GR_EDIT_PAGE (window->edit_page)))
-                go_back (window);
+        if (gr_edit_page_save (GR_EDIT_PAGE (window->edit_page))) {
+                GrRecipe *recipe;
+
+                recipe = gr_edit_page_get_recipe (GR_EDIT_PAGE (window->edit_page));
+                gr_window_show_recipe (window, recipe);
+                gr_edit_page_clear (GR_EDIT_PAGE (window->edit_page));
+        }
 }
 
 static void
@@ -164,26 +174,11 @@ stop_search (GrWindow *window)
         gr_window_go_back (window);
 }
 
-#if 0
-static void
-maybe_stop_search (GtkButton  *button,
-                   GParamSpec *pspec,
-                   GrWindow   *window)
-{
-        if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
-                stop_search (window);
-}
-#endif
-
 static void
 switch_to_search (GrWindow *window)
 {
-        save_back_entry (window);
-
-        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), _("Search"));
-
-        gtk_stack_set_visible_child_name (GTK_STACK (window->header_start_stack), "back");
-        gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "title");
+        gtk_stack_set_visible_child_name (GTK_STACK (window->header_start_stack), "main");
+        gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "main");
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_end_stack), "search");
 
         gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "search");
@@ -193,11 +188,17 @@ void
 gr_window_show_search (GrWindow   *window,
                        const char *search)
 {
+        g_auto(GStrv) terms = NULL;
+
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (window->search_button), TRUE);
         switch_to_search (window);
-        gr_query_editor_set_query (GR_QUERY_EDITOR (window->search_bar), search);
+        terms = g_new0 (char *, 2);
+        terms[0] = g_strdup (search);
+        gr_query_editor_set_terms (GR_QUERY_EDITOR (window->search_bar), (const char **)terms);
 }
 
+static void search_changed (GrWindow *window);
+
 static void
 visible_page_changed (GrWindow *window)
 {
@@ -206,7 +207,15 @@ visible_page_changed (GrWindow *window)
         visible = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
 
         if (strcmp (visible, "search") != 0) {
+                g_signal_handlers_block_by_func (window->search_bar, search_changed, window);
                 gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (window->search_button), FALSE);
+                g_signal_handlers_unblock_by_func (window->search_bar, search_changed, window);
+        }
+        if (strcmp (visible, "list") == 0) {
+                gr_list_page_repopulate (GR_LIST_PAGE (window->list_page));
+        }
+        else {
+                gr_list_page_clear (GR_LIST_PAGE (window->list_page));
         }
 
         if (strcmp (visible, "recipes") == 0) {
@@ -222,14 +231,14 @@ static void
 search_changed (GrWindow *window)
 {
         const char *visible;
-        const char *terms;
+        const char **terms;
 
         visible = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
 
         if (strcmp (visible, "search") != 0)
                 switch_to_search (window);
 
-        terms = gr_query_editor_get_query (GR_QUERY_EDITOR (window->search_bar));
+        terms = gr_query_editor_get_terms (GR_QUERY_EDITOR (window->search_bar));
         gr_search_page_update_search (GR_SEARCH_PAGE (window->search_page), terms);
 }
 
@@ -251,7 +260,6 @@ update_cooking_button (GrWindow *window,
         }
 }
 
-
 static void
 start_or_stop_cooking (GrWindow *window)
 {
@@ -411,7 +419,7 @@ gr_window_show_recipe (GrWindow *window,
 
         gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (window->search_bar), FALSE);
 
-        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), gr_recipe_get_name (recipe));
+        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), gr_recipe_get_translated_name (recipe));
 
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_start_stack), "back");
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "title");
@@ -428,7 +436,7 @@ gr_window_edit_recipe (GrWindow *window,
 
         gr_edit_page_edit (GR_EDIT_PAGE (window->edit_page), recipe);
 
-        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), gr_recipe_get_name (recipe));
+        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), gr_recipe_get_translated_name (recipe));
 
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_start_stack), "back");
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "title");
@@ -611,10 +619,14 @@ void
 gr_window_show_search_by_ingredients (GrWindow   *window,
                                       const char *ingredient)
 {
-        g_autofree char *term = NULL;
+        g_auto(GStrv) terms = NULL;
 
         switch_to_search (window);
         gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (window->search_bar), TRUE);
-        term = g_strconcat ("i+:", ingredient, NULL);
-        gr_query_editor_set_query (GR_QUERY_EDITOR (window->search_bar), term);
+
+        terms = g_new (char *, 2);
+        terms[0] = g_strconcat ("i+:", ingredient, NULL);
+        terms[1] = NULL;
+
+        gr_query_editor_set_terms (GR_QUERY_EDITOR (window->search_bar), (const char **)terms);
 }
